// (C) Copyright Exoligent 2014-2017
// Authors and Contributors: Maxime Coroyer
#ifndef _FIPCORE_H_
#define _FIPCORE_H_

#include <stdint.h>

#include "ExoErrors.h"
#include "FipAccessCommon.h"

#if defined(_WIN32) || defined(_WIN64)
#   include <windows.h>
#   if defined(FIPCORE_STATIC)
#       define FIPCORE_EXPORT
#   else
#       if defined(FIPCORE_DLL_EXPORT)
#           define FIPCORE_EXPORT __declspec(dllexport)
#       else
#           define FIPCORE_EXPORT __declspec(dllimport)
#       endif
#   endif
#elif defined(__linux__) || defined(__APPLE__) || defined(__QNX__)
#   define FIPCORE_EXPORT 
#endif

#ifdef __cplusplus
    extern "C" {
#endif

// --------------------------------------------------------------------------------------
// Exportable functions
// --------------------------------------------------------------------------------------
// --- Device Access
FIPCORE_EXPORT uint32_t FC_DEVICE_OPEN(EXO_DEVICE_TYPE device_type, uint8_t device_index);
FIPCORE_EXPORT uint32_t FC_DEVICE_CLOSE();

FIPCORE_EXPORT uint32_t FC_GET_RGB_LED( EXO_DEVICE_TYPE device_type, uint8_t device_index,
                                        LED_RGB_COLOR* led_state);
FIPCORE_EXPORT uint32_t FC_SET_RGB_LED( EXO_DEVICE_TYPE device_type, uint8_t device_index,
                                        LED_RGB_COLOR led_state);

// --- Device Info Access
FIPCORE_EXPORT uint32_t FC_GET_DEVICECONF(EXO_DEVICE_TYPE device_type,
                                        uint8_t device_index, FIP_DEVICECONF* device_conf);
FIPCORE_EXPORT uint32_t FC_GET_DEVICES_NUMBER(uint8_t* nb_detected_devices);
FIPCORE_EXPORT uint32_t FC_GET_DEVICES_LIST(FIP_DEVICECONF* devices_conf,
                                        uint8_t* nb_detected_devices);
FIPCORE_EXPORT uint32_t FC_IS_REGISTERED_DEVICE(EXO_DEVICE_TYPE device_type,
                                        uint8_t device_index, EXO_SOFT_TYPE soft_type);

// --- FIP Context Info
FIPCORE_EXPORT uint32_t FC_GET_CONF(FIP_USERCONF* fip_conf);
FIPCORE_EXPORT uint32_t FC_GET_VAR_CONF(uint16_t identifier, USERDEF_VARIABLE* var_conf);
FIPCORE_EXPORT uint32_t FC_GET_MSG_CONF(uint16_t dest_id, uint8_t dest_seg, uint16_t src_id, uint8_t src_seg, USERDEF_MESSAGE* msg_conf);
FIPCORE_EXPORT uint32_t FC_GET_TAG_CONF(uint16_t tag_id, USERDEF_TAG* tag_conf);
FIPCORE_EXPORT uint32_t FC_GET_TAG_CONF_BY_NAME(const char* tag_name, USERDEF_TAG* tag_conf);

// --- System Management
FIPCORE_EXPORT uint32_t FC_RUN(EXO_FILE_TYPE type, const char* file_path);
FIPCORE_EXPORT uint32_t FC_STOP();
FIPCORE_EXPORT uint32_t FC_GET_MEDIUM(FIP_NETWORK_STATUS* fip_network_status);
FIPCORE_EXPORT uint32_t FC_SET_MEDIUM_CMD(uint16_t fieldual_cmd);

// --- System Management - OPC UA Server
FIPCORE_EXPORT uint32_t FC_OPCUA_RUN_STARTUP(uint16_t port);
FIPCORE_EXPORT uint32_t FC_OPCUA_RUN_ITERATE(bool wait_internal, uint16_t* timeout);
FIPCORE_EXPORT uint32_t FC_OPCUA_RUN_SHUTDOWN();
FIPCORE_EXPORT uint32_t FC_OPCUA_RUN(uint16_t port, bool* running);

// --- Fip Variable Access
FIPCORE_EXPORT uint32_t FC_WRITE_VAR(uint16_t identifier, PRODUCED_VARIABLE* p_var);
FIPCORE_EXPORT uint32_t FC_READ_VAR(uint16_t identifier, CONSUMED_VARIABLE* c_var,
                                bool* is_prompt, bool* is_refreshed);
FIPCORE_EXPORT uint32_t FC_READ_VAR_SVF(uint16_t identifier, CONSUMED_VARIABLE* c_var,
                                    bool* is_prompt, bool* is_refreshed);
FIPCORE_EXPORT uint32_t FC_READ_VAR_TIME(uint16_t identifier,
                            CONSUMED_VARIABLE_TIME* c_var_time,
                            uint32_t* production_time, uint32_t* transmission_time,
                            uint32_t* consumption_time, uint32_t* reception_time,
                            bool* is_prompt, bool* is_refreshed);
FIPCORE_EXPORT uint32_t FC_READ_LSTPRESENT(PRESENT_LIST* present_list);
FIPCORE_EXPORT uint32_t FC_SEND_APER(uint16_t identifier, uint8_t mode);
FIPCORE_EXPORT uint32_t FC_PURGE_APER(uint8_t fifo_type);
FIPCORE_EXPORT uint32_t FC_DEF_VAR_EVENT(uint16_t identifier, uint16_t event_code);

// --- Fip Tag Access
FIPCORE_EXPORT uint32_t FC_WRITE_TAG(uint16_t tag_id, void* data);
FIPCORE_EXPORT uint32_t FC_WRITE_TAG_BY_NAME(const char* tag_name, void* data);
FIPCORE_EXPORT uint32_t FC_READ_TAG(uint16_t tag_id, void* data);
FIPCORE_EXPORT uint32_t FC_READ_TAG_BY_NAME(const char* tag_name, void* data);

                        
// --- Fip Event Access
FIPCORE_EXPORT uint32_t FC_READ_EVT(uint16_t* event_code, uint16_t* event_parameter);
FIPCORE_EXPORT uint32_t FC_ACQ_EOC(uint16_t ack_eoc);

// --- Fip Message Access
FIPCORE_EXPORT uint32_t FC_WRITE_MSG(uint16_t channel_number,
                                MESSAGE_FRAME_OUT* message_frame_out);
FIPCORE_EXPORT uint32_t FC_READ_MSG(MESSAGE_FRAME_IN* message_frame_in);
FIPCORE_EXPORT uint32_t FC_PURGE_MSG(uint16_t channel_number,
                                uint16_t* nb_blocks_freed, uint16_t* blocks_freed_tab);

// --- Fip Message Access - File Transfer
FIPCORE_EXPORT uint32_t FC_SET_FILE_TRANSFER_CTX(const char* input_directory, cb_progression callback_prog, cb_completion callback_comp, void* callback_ctx);
FIPCORE_EXPORT uint32_t FC_SEND_FILE(uint16_t channel_number, 
                                     uint16_t dest_id, uint8_t dest_seg,
                                     uint16_t src_id, uint8_t src_seg, const char* file_path);
FIPCORE_EXPORT uint32_t FC_ABORT_FILE_EMISSION();

// --- Fip Ba Access
FIPCORE_EXPORT uint32_t FC_RUN_BA(uint8_t ba_number);
FIPCORE_EXPORT uint32_t FC_STOP_BA();

FIPCORE_EXPORT uint32_t FC_CONTINUE_BA(uint16_t ba_continue_address);
FIPCORE_EXPORT uint32_t FC_CHANGE_MACROCYCLE(uint8_t ba_number);
FIPCORE_EXPORT uint32_t FC_SYNCHRO_BA();
FIPCORE_EXPORT uint32_t FC_CHANGE_PARAM_BA(BA_PARAMETERS* ba_parameters);

#ifdef __cplusplus
            }
#endif


#endif
